@@ -98,9 +98,19 @@ typedef __u32 __be32;
 #endif /* LINUX_2_2 */
 
 #ifdef LINUX_2_4__donotuse
+#include <linux/kernel.h>
 #include <linux/init.h>
 #include <linux/pm.h>
+#include <linux/list.h>
+#include <asm/processor.h>
 #include <asm/page.h>
+#ifndef likely
+#define likely(x)	x
+#define unlikely(x)	x
+#endif
+#ifndef cpu_relax
+#define cpu_relax()
+#endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 3)
 #define pci_set_dma_mask(pci, mask) (pci->dma_mask = mask, 0)
 #endif
@@ -114,10 +124,29 @@ typedef __u32 __be32;
 #ifndef rwlock_init
 #define rwlock_init(x) do { *(x) = RW_LOCK_UNLOCKED; } while(0)
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 9)
+#define prefetch(x)	x
+#endif
 #ifndef list_for_each_safe
 #define list_for_each_safe(pos, n, head) \
 	for (pos = (head)->next, n = pos->next; pos != (head); pos = n, n = pos->next)
 #endif
+#ifndef list_for_each_entry
+#define list_for_each_entry(pos, head, member)                          \
+        for (pos = list_entry((head)->next, typeof(*pos), member),      \
+                     prefetch(pos->member.next);                        \
+             &pos->member != (head);                                    \
+             pos = list_entry(pos->member.next, typeof(*pos), member),  \
+                     prefetch(pos->member.next))
+#endif
+#ifndef list_for_each_prev
+#define list_for_each_prev(pos, head) \
+	for (pos = (head)->prev; prefetch(pos->prev), pos != (head); \
+        	pos = pos->prev)
+#endif
+#ifndef might_sleep
+#define might_sleep() do { } while (0)
+#endif
 #endif /* LINUX_2_4__donotuse */
 
 #ifndef __devexit_p
@@ -212,8 +241,11 @@ static inline struct proc_dir_entry *PDE(const struct inode *inode)
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 9)
+#include <linux/interrupt.h>
+#ifndef in_atomic
 #define in_atomic()	in_interrupt()
 #endif
+#endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 14)
 typedef unsigned __nocast gfp_t;
@@ -663,13 +695,26 @@ static inline void *snd_compat_vmap(struct page **pages, unsigned int count, uns
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 2)
 struct class;
 struct class_device;
+struct class_device_attribute { }; /* dummy */
+#define __ATTR(cls,perm,read,write) { } /* dummy */
 static inline struct class_device* class_device_create(struct class *class, struct class_device *parent, int devnum, ...) { return NULL; }
 static inline void class_device_destroy(struct class *class, int devnum) { return; }
+static inline void *class_get_devdata(struct class_device *dev) { return NULL; }
 #else /* >= 2.6.2 */
 #ifndef CONFIG_SND_NESTED_CLASS_DEVICE
 #include <linux/device.h>
 #define class_device_create(cls,prt,devnum,args...) class_device_create(cls,devnum,##args)
 #endif
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
+#include <linux/sysfs.h>
+#ifndef __ATTR
+#define __ATTR(_name,_mode,_show,_store) { \
+	.attr = {.name = __stringify(_name), .mode = _mode, .owner = THIS_MODULE },	\
+	.show	= _show,					\
+	.store	= _store,					\
+}
+#endif /* __ATTR */
+#endif /* < 2.6.8 */
 #endif
 
 /* msleep */
@@ -744,13 +789,19 @@ static inline unsigned long msecs_to_jiffies(const unsigned int m)
 #define snd_dma_isa_data()	NULL
 #define snd_dma_sbus_data(sbus)	((struct device *)(sbus))
 #define snd_dma_continuous_data(x)	((struct device *)(unsigned long)(x))
-#ifndef DMA_32BIT_MASK
-#define DMA_32BIT_MASK 0xffffffff
-#endif
 #endif
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 17)
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)
+#include <linux/mm.h>
+#include <asm/io.h>
+#include <asm/scatterlist.h>
+#endif
 #include <linux/dma-mapping.h>
+
+#ifndef DMA_32BIT_MASK
+#define DMA_32BIT_MASK 0xffffffff
+#endif
 #ifndef DMA_31BIT_MASK
 #define DMA_31BIT_MASK	0x000000007fffffffULL
 #endif
@@ -1174,4 +1225,65 @@ static inline int snd_pnp_register_card_driver(struct pnp_card_driver *drv)
  */
 #define OPL3_HW_OPL3_PC98	0x0305	/* PC9800 */
 
+/*
+ * IRQF_* flags
+ */
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17)
+#include <linux/interrupt.h>
+#ifndef IRQF_SHARED
+#include <linux/signal.h>
+#define IRQF_SHARED			SA_SHIRQ
+#define IRQF_DISABLED			SA_INTERRUPT
+#define IRQF_SAMPLE_RANDOM		SA_SAMPLE_RANDOM
+#define IRQF_PERCPU			SA_PERCPU
+#ifdef SA_PROBEIRQ
+#define IRQF_PROBE_SHARED		SA_PROBEIRQ
+#else
+#define IRQF_PROBE_SHARED		0 /* dummy */
+#endif
+#endif /* IRQ_SHARED */
+#endif /* <= 2.6.17 */
+
+/*
+ * lockdep macros
+ */
+#if LINUX_VERSION_CODE <= KERNEL_VERSION(2, 6, 17)
+#define lockdep_set_class(lock, key)		do { (void)(key); } while (0)
+#define down_read_nested(sem, subclass)		down_read(sem)
+#define down_write_nested(sem, subclass)	down_write(sem)
+#define down_read_non_owner(sem)		down_read(sem)
+#define up_read_non_owner(sem)			up_read(sem)
+#endif
+
+/*
+ * PPC-specfic
+ */
+#ifdef CONFIG_PPC
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,18)
+#include <linux/interrupt.h>
+#ifndef NO_IRQ
+#define NO_IRQ	(-1)
+#endif
+#define irq_of_parse_and_map(node, x) \
+	(((node) && (node)->n_intrs > (x)) ? (node)->intrs[x].line : NO_IRQ)
+#endif /* < 2.6.18 */
+#endif /* PPC */
+
+/* MSI */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 8)
+static inline int snd_pci_enable_msi(struct pci_dev *dev) { return -1; }
+#undef pci_enable_msi
+#define pci_enable_msi(dev) snd_pci_enable_msi(dev)
+#undef pci_disable_msi
+#define pci_disable_msi(dev)
+#endif
+
+/* SEEK_XXX */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 18)
+#include <linux/fs.h>
+#define SEEK_SET	0
+#define SEEK_CUR	1
+#define SEEK_END	2
+#endif
+
 #endif /* __SOUND_LOCAL_DRIVER_H */
