@@ -19,7 +19,9 @@
 /* include linux/usb.h at first since it defines another compatibility layers, which
  * conflicts with ours...
  */
+#define device compat_net_device
 #include <linux/usb.h>
+#undef device
 #undef IORESOURCE_IO
 #endif
 
@@ -69,6 +71,17 @@ static __inline__ void list_del_init(struct list_head *entry)
 	for (pos = (head)->next, n = pos->next; pos != (head); \
 		pos = n, n = pos->next)
 
+/**
+ * list_for_each_entry	-	iterate over list of given type
+ * @pos:	the type * to use as a loop counter.
+ * @head:	the head for your list.
+ * @member:	the name of the list_struct within the struct.
+ */
+#define list_for_each_entry(pos, head, member)				\
+	for (pos = list_entry((head)->next, typeof(*pos), member);	\
+	     pos->member.next, &pos->member != (head); 	\
+	     pos = list_entry(pos->member.next, typeof(*pos), member))
+
 /* rw_semaphore - replaced with mutex */
 #define rw_semaphore semaphore
 #define init_rwsem(x) init_MUTEX(x)
@@ -587,4 +600,6 @@ static inline int abs(int val)
 #define cpu_relax()
 #endif
 
+#define might_sleep() do { } while (0)
+
 #endif /* <2.3.0 */
